unit uInspector;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ExtCtrls, ComCtrls, Buttons, opcGroupdlg,
  opcda,dOPCBrowseDlg,inifiles,
  ImgList, dOPC;

type
  TInspecDlg = class(TForm)
    Panel2: TPanel;
    TreeView1: TTreeView;
    ItemView: TListView;
    Splitter1: TSplitter;
    GroupPopup: TPopupMenu;
    AddGroup1: TMenuItem;
    RemoveGroup1: TMenuItem;
    Properties1: TMenuItem;
    ItemPopup: TPopupMenu;
    A1: TMenuItem;
    RemoveItem1: TMenuItem;
    ImageList1: TImageList;
    Active1: TMenuItem;
    WriteValue1: TMenuItem;
    RemoveallItems1: TMenuItem;
    RemoveallGroups1: TMenuItem;
    Label7: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    LName: TLabel;
    LUpdate: TLabel;
    LDeadBand: TLabel;
    LActive: TLabel;
    AsyncWriteValue1: TMenuItem;

    procedure FormShow(Sender: TObject);
    procedure AddGroup1Click(Sender: TObject);
    procedure RemoveGroup1Click(Sender: TObject);
    procedure Properties1Click(Sender: TObject);
    procedure A1Click(Sender: TObject);
    procedure ItemPopupPopup(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Active1Click(Sender: TObject);
    procedure RemoveItem1Click(Sender: TObject);
    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure RemoveallGroups1Click(Sender: TObject);
    procedure RemoveallItems1Click(Sender: TObject);
    procedure WriteValue1Click(Sender: TObject);
    procedure AsyncWriteValue1Click(Sender: TObject);

  private
    IniFile : TIniFile;
    FOPCServer: TdOPCServer;
    procedure ClearInfoLabel;
    function GetSelectedGroup: tdOPCGroup;
    procedure AddNewItems(Group: TdOPCGroup;ItemIds: TStrings);


  public
    procedure ShowGroupState;
    procedure Refresh;
    procedure OnDatachange(Sender: TObject;ItemList: TdOPCItemList);
    procedure AddItemsToGroup(GroupName: string; ItemsIds: TStrings);
    property  OPCServer: TdOPCServer read FOPCServer write FOPCServer ;
    procedure GetGroupNames(GroupNames: TStrings);
  end;

var
  InspecDlg: TInspecDlg;

implementation

{$R *.DFM}

procedure UpdateListViewItem(ListItem: TListItem; OPCItem: TdOPCItem);
begin
  ListItem.SubItems[2] := OPCItem.ValueStr;   // Value as string;
  ListItem.SubItems[3] := TimeToStr(OPCItem.TimeStamp); // TimeStamp
  if OPCItem.Quality = OPC_QUALITY_GOOD then
    ListItem.SubItems[4] := 'good'                      // Quality
  else
    ListItem.SubItems[4] := 'bad';                      // Quality
end;


procedure ShowListViewItem(ListItem: TListItem; OPCItem: TdOPCItem);
begin
  ListItem.SubItems.Clear;
  ListItem.Caption := OPCItem.ItemId;
  ListItem.SubItems.Add(OPCItem.AccessPath);
  if OPCItem.IsActive then
   ListItem.SubItems.Add('X')
  else
   ListItem.SubItems.Add(' ');
  ListItem.SubItems.Add('-');   // Value
  ListItem.SubItems.Add('-');   // TimeStamp
  ListItem.SubItems.Add('-');   // Quality
end;


procedure TInspecDlg.FormCreate(Sender: TObject);
begin
  IniFile := TIniFile.Create(ChangeFileExt(Application.ExeName,'.ini'));
end;

procedure TInspecDlg.FormDestroy(Sender: TObject);
begin
  IniFile.free;
end;

procedure TInspecDlg.FormShow(Sender: TObject);
begin
  ItemView.Items.Clear;
  Treeview1.Items.Clear;
  ClearInfoLabel;
end;


procedure TInspecDlg.ClearInfoLabel;
var
  i : integer;
  C : TComponent;
begin
  for i := 0 to ComponentCount-1 do        // clears all Infolabels
  begin
    C := Components[i];
    if (C.Tag = 1) and (C is TLabel) then
      TLabel(C).Caption := '';
  end;
end;

function TInspecDlg.GetSelectedGroup: tdOPCGroup;
var
  Node : TTreeNode;
begin
  result := nil;
  Node := TreeView1.Selected;
  if Node <> nil then
    result := OPCServer.OPCGroups.GetOPCGroup(Node.Text); // get Group
end;



procedure TInspecDlg.ShowGroupState;
var
  Group: tdOPCGroup;
  State: TGroupStateRec;
begin
  if OPCServer.Active then
  begin
      Group := GetSelectedGroup;  // selected Group from Treeview
      if Group <> nil then
      begin
         Group.GetState(State); // get state direct from OPC Server
         LName.Caption     := State.Name;
         LUpdate.Caption   := IntToStr(State.UpdateRate);
         LDeadBand.Caption := format('%.2f',[State.DeadBand]);
         if State.IsActive then
           LActive.Caption := 'active'
         else
           LActive.Caption := 'not active';
      end
      else ClearInfoLabel;
  end
  else
    ClearInfoLabel;
end;


procedure TInspecDlg.AddGroup1Click(Sender: TObject);
var
  GroupParams : tGroupStateRec;
  Node : TTreeNode;
begin
  fillchar(GroupParams,sizeof(GroupParams),0);
  GroupParams.UpdateRate := OPCServer.OPCGroupDefault.UpdateRate;
  GroupParams.IsActive   := OPCServer.OPCGroupDefault.IsActive;
  GroupParams.LocaleId   := OPCServer.OPCGroupDefault.LocaleId;
  GroupParams.DeadBand   := OPCServer.OPCGroupDefault.DeadBand;
  GroupParams.TimeBias   := OPCServer.OPCGroupDefault.TimeBias;
  if OPCEditGroupDialog(GroupParams) then
  begin
    OPCServer.OPCGroupDefault.UpdateRate := GroupParams.UpdateRate;
    OPCServer.OPCGroupDefault.IsActive   := GroupParams.IsActive;
    OPCServer.OPCGroupDefault.LocaleId   := GroupParams.LocaleId;
    OPCServer.OPCGroupDefault.DeadBand   := GroupParams.DeadBand;
    OPCServer.OPCGroupDefault.TimeBias   := GroupParams.TimeBias ;
    OPCServer.OPCGroups.Add(GroupParams.Name);      // add to  OPC Server
    Node := TreeView1.Items.AddChild(nil,GroupParams.Name); // add to Tree
    Node.Selected := true;
    Node.Focused  := true;
  end;
end;

procedure TInspecDlg.Refresh;
var
  Group : tdOPCGroup;
  i : integer;
begin
  ItemView.Items.Clear;   // delete all visible Items
  Treeview1.Items.Clear;
  for i := 0 to OPCServer.OPCGroups.Count-1 do
  begin
    Group := OPCServer.OPCGroups[i];
    TreeView1.Items.AddChild(nil,Group.Name); // add to Tree
  end;
  if TreeView1.Items.Count > 0 then
  begin
    TreeView1.Items[0].Selected := true;
    TreeView1.Items[0].Focused  := true;
    TreeView1Change(self, TreeView1.Items[0]);
  end;
  ShowGroupState;
end;

procedure TInspecDlg.RemoveGroup1Click(Sender: TObject);
var
  Node : TTreeNode;
begin
  Node := TreeView1.Selected;
  if Node <> nil then
  begin
    OPCServer.OPCGroups.Remove(Node.Text);  // delete from Server
    ItemView.Items.Clear;
    Treeview1.Items.Delete(Node);           // delete from Tree
  end;
end;


procedure TInspecDlg.RemoveallGroups1Click(Sender: TObject);
begin
  OPCServer.OPCGroups.RemoveAll;  // delete from Server
  ItemView.Items.Clear;
  Treeview1.Items.Clear;                // delete from Server
end;


procedure TInspecDlg.Properties1Click(Sender: TObject);
var
  Group : TdOPCGroup;
  Node : TTreeNode;
  GroupParams : tGroupStateRec;
begin
  Node := TreeView1.Selected;
  if Node <> nil then
  begin
    Group := OPCServer.OPCGroups.GetOPCGroup(Node.Text); // get Group
    if Group <> nil then
    begin
      GroupParams.Name       := Group.Name;
      GroupParams.UpdateRate := Group.UpdateRate;
      GroupParams.IsActive   := Group.IsActive;
      GroupParams.LocaleId   := Group.LocaleId;
      GroupParams.DeadBand   := Group.DeadBand;
      GroupParams.TimeBias   := Group.TimeBias;
      if OPCEditGroupDialog(GroupParams) then
      begin
        Group.Name       := GroupParams.Name;
        Node.Text        := GroupParams.Name;
        Group.UpdateRate := GroupParams.UpdateRate;
        Group.IsActive   := GroupParams.IsActive;
        Group.LocaleId   := GroupParams.LocaleId;
        Group.DeadBand   := GroupParams.DeadBand;
        Group.TimeBias   := GroupParams.TimeBias;
      end;
    end
    else ShowMessage('ERROR');
  end
  else ShowMessage('First select Group');
end;


procedure TInspecDlg.AddNewItems(Group: TdOPCGroup;ItemIds: TStrings);
var
  Item    : TListItem;
  OPCItem : tdOpcItem;
  i       : integer;
begin
  if Group <> nil then
  begin
    for i := 0 to ItemIds.Count-1 do
    begin
      Group.DefaultItem.IsActive := true;
      OPCItem := Group.OPCItems.AddItem(ItemIds[i]); // add new Item
      if GetSelectedGroup = Group then
      begin
        Item := ItemView.Items.Add;                    // add new viewitem
        OPCItem.Data := Item;                          // set pointer to view
        Item.Data := OPCItem;                          // set pointer to OpcItem
        Item.Caption := OPCItem.ItemId;
        ShowListViewItem(Item,OPCItem);
      end;
    end;
  end;
end;

procedure TInspecDlg.A1Click(Sender: TObject);
var
  ItemIds: TStringlist;
begin
    ItemIds := TStringlist.create;
    try
       if OPCBrowseDialog(OPCServer.Browser, ItemIds) then
       begin
         AddNewItems(GetSelectedGroup,ItemIds);
       end
    finally
      ItemIds.Free;
    end;
end;

procedure TInspecDlg.ItemPopupPopup(Sender: TObject);
var
 Enable : boolean;
 i      : integer;
 ListItem : TListItem;
begin
  Enable :=  GetSelectedGroup <> nil;         // only Enable if group selected
  for i := 0 to ItemPopup.Items.Count -1 do
     ItemPopup.Items[i].Enabled := Enable;
  if Enable then
  begin
     ListItem := ItemView.Selected;
     if ListItem <> nil then
     begin
       Active1.Checked := tdOPCItem(ListItem.Data).IsActive;
     end;
  end;
end;

procedure TInspecDlg.OnDatachange(Sender: TObject; ItemList: TdOPCItemList);
var
  ListItem : TListItem;
  OPCItem  : tdOpcItem;
  i        : integer;
begin
  for i := 0 to ItemList.Count-1 do
  begin
     OPCItem  := ItemList[i];
     ListItem := OPCItem.Data;
     if ListItem <> nil then
       UpdateListViewItem(ListItem, OPCItem);
  end;
end;


procedure TInspecDlg.Active1Click(Sender: TObject);
var
  ListItem : TListItem;
begin
  ListItem := ItemView.Selected;
  if ListItem <> nil then
  begin
    tdOPCItem(ListItem.Data).IsActive := not Active1.Checked;
    ShowListViewItem(ListItem,tdOPCItem(ListItem.Data));
    UpdateListViewItem(ListItem,tdOPCItem(ListItem.Data));
  end;
end;

procedure TInspecDlg.RemoveItem1Click(Sender: TObject);
var
  ListItem : TListItem;
begin
  ListItem := ItemView.Selected;
  if ListItem <> nil then
  begin
    tdOPCItem(ListItem.Data).Free;
    ListItem.Free;
  end;
end;



procedure TInspecDlg.TreeView1Change(Sender: TObject; Node: TTreeNode);
var
  ListItem : TListItem;
  OPCItem  : TdOpcItem;
  Group    : tdOPCGroup;
  i        : integer;
begin
  Group := GetSelectedGroup;

  for i := 0 to Itemview.Items.Count-1 do       // we want to clear Listview
  begin                                         // so we have to clear pointer
     ListItem := Itemview.Items[i];             // on OPCItems
     tdOPCItem(ListItem.Data).Data := nil;
  end;
  Itemview.Items.Clear;                         // Clear Items in Listview

  if Group = nil then exit;

  for i := 0 to Group.OPCItems.Count-1 do       // now get the new Items
  begin
    OPCItem  := Group.OPCItems[i];
    ListItem := ItemView.Items.Add;             // add new viewitem
    OPCItem.Data     := ListItem;               // set pointer to view
    ListItem.Data    := OPCItem;                // set pointer to OpcItem
    ShowListViewItem(ListItem,OPCItem);
    UpdateListViewItem(ListItem,OPCItem);
  end;
end;

procedure TInspecDlg.RemoveallItems1Click(Sender: TObject);
var
  Group    : tdOPCGroup;
begin
  Group := GetSelectedGroup;
  if Group <> nil then
  begin
     Group.OPCItems.RemoveAll;
     ItemView.Items.Clear;
  end;
end;

procedure TInspecDlg.AddItemsToGroup(GroupName: string; ItemsIds: TStrings);
begin
  AddNewItems(OPCServer.OPCGroups.GetOPCGroup(GroupName), ItemsIds);
end;

procedure TInspecDlg.GetGroupNames(GroupNames: TStrings);
var
  i : integer;
begin
  GroupNames.Clear;
  for i := 0 to Treeview1.Items.Count-1 do
  begin
     GroupNames.Add(Treeview1.Items[i].Text);
  end;
end;

procedure TInspecDlg.WriteValue1Click(Sender: TObject);
var
  NewValue : string;
  ListItem : TListItem;
begin
 ListItem := ItemView.Selected;
 if ListItem <> nil then
 begin
   NewValue := '';
   if InputQuery('Write new Value','new Value',NewValue) then
     tdOPCItem(ListItem.Data).WriteSync(NewValue)
 end;
end;

procedure TInspecDlg.AsyncWriteValue1Click(Sender: TObject);
var
  NewValue : string;
  ListItem : TListItem;
begin
 ListItem := ItemView.Selected;
 if ListItem <> nil then
 begin
   NewValue := '';
   if InputQuery('Write new Value','new Value',NewValue) then
     tdOPCItem(ListItem.Data).WriteASync(NewValue)
 end;
end;

end.
